import { defineCommand, runMain as runMain$1 } from 'citty';
import { consola } from 'consola';
import { mkdir, copyFile } from 'node:fs/promises';
import { globby } from 'globby';
import { join, dirname, resolve, basename } from 'pathe';
import { fileURLToPath } from 'url';
import { readPackageJSON, writePackageJSON } from 'pkg-types';
import { installDependencies } from 'nypm';
import { stat } from 'fs/promises';

const $logger = consola;

async function copy(options) {
  const files = await globby(options.patterns, {
    cwd: options.src,
    dot: true
  });
  return Promise.all(
    files.map(async (p) => {
      const from = join(options.src, p);
      const to = join(options.target, p);
      await mkdir(dirname(to), { recursive: true });
      return copyFile(from, to);
    })
  );
}

function getPkgManager() {
  const userAgent = process.env.npm_config_user_agent || "";
  if (userAgent.startsWith("yarn")) {
    return "yarn";
  }
  if (userAgent.startsWith("pnpm")) {
    return "pnpm";
  }
  if (userAgent.startsWith("bun")) {
    return "bun";
  }
  return "npm";
}

async function createProject(options) {
  await installTemplate(options);
}
async function installTemplate(options) {
  $logger.info(`Installing application template '${options.template}'...`);
  const templatePath = fileURLToPath(
    new URL(`../templates/${options.template}`, import.meta.url)
  );
  await copy({
    src: templatePath,
    target: options.path,
    patterns: ["**"]
  });
  const pkg = await readPackageJSON(templatePath);
  pkg.name = options.projectName;
  pkg.scripts = {
    prepare: "osmos prepare",
    dev: "osmos dev",
    build: "osmos build"
  };
  pkg.devDependencies = {
    ...pkg.devDependencies,
    typescript: "^5",
    "@types/node": "^20",
    "@types/react": "^19",
    "@types/react-dom": "^19"
  };
  pkg.dependencies = {
    ...pkg.dependencies,
    "@osmosjs/osmos": "latest",
    // TODO: Fix version instead
    react: "^19",
    "react-dom": "^19"
  };
  await writePackageJSON(join(options.path, "package.json"), pkg);
  $logger.info(`Installing dependencies`);
  await installDependencies({
    packageManager: getPkgManager(),
    cwd: options.path
  });
}

async function askProjectName(cwd) {
  const value = await $logger.prompt(
    "Where should we create your new project?",
    {
      placeholder: "./future-unicorn",
      required: true
    }
  );
  if (value === void 0) {
    return askProjectName(cwd);
  }
  if (typeof value !== "string") {
    process.exit(1);
  }
  const path = join(cwd, value);
  if (await stat(path).then(() => true).catch(() => false)) {
    $logger.error(`The directory '${path}' is not empty`);
    return askProjectName(cwd);
  }
  return {
    path,
    projectName: basename(path)
  };
}
async function askEslint() {
  const value = await $logger.prompt("Should we configure ESLint for you?", {
    type: "confirm"
  });
  if (typeof value !== "boolean") {
    process.exit(1);
  }
  return value;
}
async function askTailwind() {
  const value = await $logger.prompt("What about TailwindCSS?", {
    type: "confirm"
  });
  if (typeof value !== "boolean") {
    process.exit(1);
  }
  return value;
}
const main = defineCommand({
  meta: {
    name: "create-osmos"
  },
  args: {
    cwd: {
      type: "string",
      description: "Specify the working directory",
      valueHint: "directory",
      default: "."
    }
  },
  async run({ args }) {
    const cwd = resolve(args.cwd);
    const { path, projectName } = await askProjectName(cwd);
    const eslint = await askEslint();
    const tailwind = await askTailwind();
    await createProject({
      path,
      projectName,
      eslint,
      tailwind,
      template: "app"
    });
  }
});

function runMain() {
  return runMain$1(main);
}

export { runMain };
